<html py:extends="master.xhtml" py:strip="True">
  <py:block name="title">
    Privacy Policy
  </py:block>
  <body py:block="body" py:strip="True">
    <div class="container">
      <h1>Privacy Policy</h1>

      <p>Last updated February 21, 2022</p>

      <h2>Information that is gathered from visitors</h2>
      <p>
        Log files are stored on the web server saving details such as
        the visitor's IP address, browser type, referring page and time
        of visit.
      </p>

      <p>
        Cookies may be used to remember visitor preferences when
        interacting with the website.
      </p>

      <p>
        Where registration is required, the visitor's email and a
        username will be stored on the server.  All data uploaded by the
        visitor is stored on the server.
      </p>

      <h2>How the Information is used</h2>
      <p>
        The instructor of your course uses the competition data to
        provide you with a grade on the assignment.
      </p>

      <p>
        The input files, output files, and verifications you provide on
        the website will be publicly accessible to other visitors
        without authentication.
      </p>

      <p>
        E-mail addresses are kept private, and will not be sold, rented or
        leased to third parties.
      </p>


      <h2>Cookies</h2>
      <p>
        Cookies are small digital signature files that are stored by
        your web browser that allow your preferences to be recorded when
        visiting the website.
      </p>

      <p>
        Cookies are used by this website are only used to record your
        login session.
      </p>

      <p>
        Third party cookies are not used by this website.
      </p>
    </div>
  </body>
</html>
