<html py:extends="master.xhtml" py:strip="True">
  <py:block name="title">
    Competitions Archive
  </py:block>
  <body py:block="body" py:strip="True">
    <div class="container">
      <py:for each="yeargroup in competitions">
        <h2 class="display-4 mb-2">${yeargroup.year}</h2>
        <div class="list-group">
          <py:for each="c in yeargroup.competitions">
            <a class="list-group-item"
               href="${tg.url('/competition/{}'.format(c.id))}">
              <h4>
                ${c.name}
              </h4>
              <span>Ended ${h.ftime(c.end)}</span>
            </a>
          </py:for>
        </div>
      </py:for>
    </div>
  </body>
</html>
