<html py:extends="master.xhtml" py:strip="True">
  <py:block name="title">
    Competitions List
  </py:block>
  <body py:block="body" py:strip="True">
    <div class="container">
      <h1 class="mb-3">Active Competitions</h1>
      <py:if test="competitions">
        <div class="list-group">
          <py:for each="c in competitions">
            <a class="list-group-item"
               href="${tg.url('/competition/{}'.format(c.id))}">
              <h4>
                ${c.name}
              </h4>
              <span>${h.ftime(c.input_upload_begins, c.end)}</span>
            </a>
          </py:for>
        </div>
      </py:if><py:else>
        <p>
          There are no active competitions.  You may wish to check out the
          <a href="${tg.url('/competition/archive')}">archive</a>.
        </p>
      </py:else>
    </div>
  </body>
</html>
