<html py:extends="master.xhtml" py:strip="True">
  <py:block name="title">
    Home
  </py:block>
  <body py:block="body" py:strip="True">
    <py:if test="asbool(tg.config['site.edu'])">
      <py:include href="algobowl.templates.edu_landing" />
    </py:if><py:else>
      <py:include href="algobowl.templates.nonedu_landing" />
    </py:else>
  </body>
</html>
