<div class="input-upload-stage" py:if="stage == 'input_upload'">
  <h1>Input Upload</h1>
  <p class="lead">
    Welcome to ${tg.config['site.branding.name']}! You will have until
    <strong>${h.ftime(competition.input_upload_ends)}</strong> to submit
    your input and decide upon a team name.
  </p>
  <form method="POST"
        action="${tg.url(f'/group/{group.id}/input_upload')}"
        enctype="multipart/form-data">
    <div class="form-group">
      <label for="input-upload">Input Upload</label>
      <input type="file" class="form-control-file" id="input-upload" name="input_upload" />
      <small class="form-text text-muted" py:if="group.input and not group.input.is_default">
        An input has already been uploaded. You may wish to
        <a href="${group.input.url}" download="${group.input.filename}">download your current input</a>.
      </small>
    </div>
    <div class="form-group">
      <label for="team-name">Team Name</label>
      <input type="text" class="form-control" id="team-name" name="team_name" value="${group.name}" />
      <small class="form-text text-muted">
        Please avoid names with offensive content, profanity, or are
        hard to pronounce.
        <py:if test="asbool(tg.config['site.edu'])">
          Your instructor
        </py:if><py:else>
          The site administrator
        </py:else>
        reserves the right to change your team name.
      </small>
    </div>
    <button type="submit" class="btn btn-primary">Submit</button>
  </form>
</div>
