<div class="container">
  <div class="jumbotron">
    <h1 class="display-4">Welcome to ${tg.config['site.branding.name']}</h1>
    <p class="lead">Students compete in groups to create the best heuristic to an NP-hard problem.</p>
    <hr />
    <div class="compete-link row">
      <div class="col-sm text-center">
        <h4 class="font-weight-light">Are you enrolled?</h4>
        <a class="btn btn-primary btn-lg" href="${tg.url('/group')}" role="button">
          <i class="fas fa-users fa-fw fa-lg"></i>
          Enter the Competition
        </a>
      </div>
      <div class="col-sm text-center">
        <h4 class="font-weight-light">Came to watch?</h4>
        <a class="btn btn-primary btn-lg" href="${tg.url('/competition')}" role="button">
          <i class="fas fa-trophy fa-fw fa-lg"></i>
          View the Leaderboards
        </a>
      </div>
    </div>
  </div>
  <h1>How it Works</h1>
  <p>
    Students are placed in groups and receive an NP-hard problem. They
    are asked to design a good heuristic to the problem, create an
    input to the problem, and a program which, given an input and output,
    will verify if the output is a valid solution to the input.
  </p>
  <div class="row">
    <div class="col-md">
      <div class="card">
        <div class="card-body">
          <h5 class="card-title">
            Stage 1: Input Upload
          </h5>
          <i class="fas fa-file-upload fa-fw fa-4x fa-pull-left"></i>
          <p class="card-text">
            Groups upload their inputs to the problem and select a team name.
          </p>
        </div>
      </div>
    </div>
    <div class="col-md">
      <div class="card">
        <div class="card-body">
          <h5 class="card-title">
            Stage 2: Output Upload
          </h5>
          <p class="card-text">
            <i class="fas fa-cogs fa-fw fa-4x fa-pull-left"></i>
            Groups run all inputs on their solution program and upload
            their outputs to the site.
          </p>
        </div>
      </div>
    </div>
    <div class="col-md">
      <div class="card">
        <div class="card-body">
          <h5 class="card-title">
            Stage 3: Verification
          </h5>
          <p class="card-text">
            <i class="fas fa-check fa-fw fa-4x fa-pull-left"></i>
            Groups verify that each solution to their input from all other
            groups is valid.
          </p>
        </div>
      </div>
    </div>
  </div>
  <div class="row mt-3">
    <div class="col-md">
      <div class="card">
        <div class="card-body">
          <h5 class="card-title">
            Stage 4: Resolution
          </h5>
          <p class="card-text">
            <i class="fas fa-sad-tear fa-fw fa-4x fa-pull-left"></i>
            Groups with rejections may upload new outputs for
            a penalty, or protest rejections.
          </p>
        </div>
      </div>
    </div>
    <div class="col-md">
      <div class="card">
        <div class="card-body">
          <h5 class="card-title">
            Stage 5: Open Verification
          </h5>
          <p class="card-text">
            <i class="fas fa-check-double fa-fw fa-4x fa-pull-left"></i>
            All outputs for all inputs become available, and teams may
            protest any verification.
          </p>
        </div>
      </div>
    </div>
    <div class="col-md">
      <div class="card">
        <div class="card-body">
          <h5 class="card-title">
            Stage 6: Evaluation
          </h5>
          <p class="card-text">
            <i class="fas fa-sliders-h fa-fw fa-4x fa-pull-left"></i>
            Everyone evaluates their teammates effort to be considered
            in their grade.
          </p>
        </div>
      </div>
    </div>
  </div>
  <h2 class="mt-4 mb-3"><i class="fas fa-clipboard-list fa-fw"></i> Grading</h2>
  <p>
    Grades are assigned according to the following percentages:
  </p>

  <table class="table">
    <tbody>
      <tr>
        <th scope="row">Input Uploaded</th>
        <td>5%</td>
      </tr>
      <tr>
        <th scope="row">Input Difficulty</th>
        <td>10%</td>
      </tr>
      <tr>
        <th scope="row">Valid Outputs</th>
        <td>50%</td>
      </tr>
      <tr>
        <th scope="row">Verification Accuracy</th>
        <td>20%</td>
      </tr>
      <tr>
        <th scope="row">Rankings</th>
        <td>
          15%, assigned by rank within benchmark:
          <ul>
            <li>Below benchmark 1: Between 0% and 5%</li>
            <li>Below benchmark 2: Between 5% and 10%</li>
            <li>Above benchmark 2: Between 10% and 15%</li>
          </ul>
        </td>
      </tr>
    </tbody>
  </table>

  <p>
    This rubric is designed in such a way that approximately 80% of the grade
    can be received without a competitive solution.
  </p>

  <p>
    Your individualized grade is a scaled version of your group grade
    based on evaluation responses from others in your group.
  </p>
  <h2 class="mt-4 mb-3"><i class="fas fa-stopwatch fa-fw"></i> Deadlines</h2>
  <p>
    All deadlines for each stage of the competition are strict.  Since this is a
    whole-class project, we need input upload to be complete before output
    upload may begin, we need output upload to be complete before verification
    can begin, etc.  Students are encouraged to upload early to avoid missing
    deadlines.  The site will not allow you to participate in a competition
    stage after it ends.
  </p>
  <p>
    If you miss a deadline due to an approved absence, you should contact your
    instructor to provide an alternative activity for that portion of the grade.
  </p>
</div>
