<html py:extends="master.xhtml" py:strip="True">
  <py:block name="title">
    Terms of Service
  </py:block>
  <body py:block="body" py:strip="True">
    <div class="container">
      <h1>Terms of Service</h1>

      <p>Last updated February 21, 2022</p>

      <ul>
        <li>
          The site administrator reserves the right to terminate the
          account of any user, regardless of reason.
        </li>
        <li>
          If you have any questions, please contact your course
          instructor or the site administrator.
        </li>
      </ul>
    </div>
  </body>
</html>
